# RSpec clone [![Home](https://img.shields.io/badge/Home-r--spec.dev-00af8b)](https://r-spec.dev/)

> A small [RSpec](https://github.com/rspec) clone based on [Fix testing tools for Ruby](https://github.com/fixrb).

## Warning ⚠️

I'd like to avoid confusion in the Ruby community by emphasizing that the gem of this project is not [rspec](https://rubygems.org/gems/rspec) but [r_spec](https://rubygems.org/gems/r_spec).

The [r_spec](https://rubygems.org/gems/r_spec) gem is a minimalist and personal reimplementation of the popular RSpec framework. My reimplementation is therefore independent of the original RSpec project.

![On the job](https://github.com/cyril/r_spec-clone.rb/raw/main/img/on-the-job.jpg)

For the purpose of the DSL reimplementation, I have kept the same `RSpec` module name, and following the [Ruby gems naming convention](https://guides.rubygems.org/name-your-gem/#use-underscores-for-multiple-words), I called its gem [r_spec](https://rubygems.org/gems/r_spec).

However, to go further and explicitly indicate that this reimplementation is a clone, I have grouped all the code in a `RSpec::Clone` submodule within its [r_spec-clone](https://rubygems.org/gems/r_spec-clone) gem.

## Why did I redo that?

An urge to simplify the RSpec DSL and reduce the complexity of the code that implements it, and nothing better to do during a stay in Tokyo.

## Installation

Add this line to your application's Gemfile:

```ruby
gem "r_spec"
```

And then execute:

```sh
bundle install
```

Or install it yourself as:

```sh
gem install r_spec
```

## Update

All the code is centralized in the `RSpec::Clone` module, whose source code is versioned at <https://github.com/cyril/r_spec-clone.rb>.

To download the latest updates of the [r_spec-clone](https://rubygems.org/gems/r_spec-clone) gem, execute this command:

```sh
bundle update r_spec
```

Or update it yourself as:

```sh
gem update r_spec
```

## Usage

Let's imagine a `string_hello_world_spec.rb` file that contains this code:

```ruby
require "r_spec"

app = "Hello, World!"

RSpec.describe String do
  subject do
    app
  end

  before do
    subject.gsub!("World", friend)
  end

  context "when Alice is greeted" do
    let(:friend) { "Alice" }

    it { is_expected.to eq "Hello, Alice!" }
  end

  context "when Bob is greeted" do
    let(:friend) { "Bob" }

    it { is_expected.to eq "Hello, Bob!" }
  end
end
```

We can run it with Ruby:

```sh
ruby string_hello_world_spec.rb
```

The report of the execution should be:

```txt
string_hello_world_spec.rb:19
Success: expected to eq "Hello, Alice!".
string_hello_world_spec.rb:25
Success: expected to eq "Hello, Bob!".
```

## External links

* [Official RSpec project](https://rspec.info/)
* [Personal RSpec reimplementation](https://r-spec.dev/)

## Versioning

__RSpec__ uses [Semantic Versioning 2.0.0](https://semver.org/)

## License

The [gem](https://rubygems.org/gems/r_spec) is available as open source under the terms of the [MIT License](https://opensource.org/licenses/MIT).
